﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMachine
{
    internal abstract class AbstractMap
    {
        private IDrawningObject _drawningObject = null;
        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;

        public Bitmap CreateMap(int width, int height, IDrawningObject drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }

        public (int LeftCoord, int TopCoord, int RightCoord, int BottomCoord) GetCoord()
        {
            return ((int)(_drawningObject.GetCurrentPosition().Left / _size_x), (int)(_drawningObject.GetCurrentPosition().Top / _size_y), (int)(_drawningObject.GetCurrentPosition().Right / _size_x), (int)(_drawningObject.GetCurrentPosition().Bottom / _size_y));
        }

        public bool CheckMove(Direction dir)
        {
            switch (dir)
            {
                case Direction.Left:
                    if (GetCoord().LeftCoord <= 0)
                    {
                        return false;
                    }
                    break;
                case Direction.Right:
                    if (GetCoord().RightCoord + 1 > _map.GetLength(1))
                    {
                        return false;
                    }
                    break;
                case Direction.Up:
                    if (GetCoord().TopCoord <= 0)
                    {
                        return false;
                    }
                    break;
                case (Direction.Down):
                    if (GetCoord().BottomCoord -1 > _map.GetLength(0))
                    {
                        return false;
                    }
                    break;
            }
            switch (dir)
            {
                case Direction.Left:
                for (int i = GetCoord().TopCoord; i <= GetCoord().BottomCoord; i++)
                {
                    if (_map[GetCoord().LeftCoord - (int)((_drawningObject.Step) / _size_x) - 1, i] == _barrier)
                        return false;
                }
                    break;
                case Direction.Right:
                    for (int i = GetCoord().TopCoord; i < GetCoord().BottomCoord; i++)
                    {
                        if (_map[GetCoord().RightCoord + (int)((_drawningObject.Step) / _size_x) + 1, i] == _barrier)
                            return false;
                    }
                    break;
                case Direction.Up:
                    for (int i = GetCoord().LeftCoord; i < GetCoord().RightCoord; i++)
                    {
                        if (_map[i, GetCoord().TopCoord - (int)((_drawningObject.Step) / _size_y) - 1] == _barrier)
                            return false;
                    }
                    break;
                case Direction.Down:
                    for (int i = GetCoord().LeftCoord; i < GetCoord().RightCoord; i++)
                    {
                        if (_map[i, GetCoord().BottomCoord + (int)((_drawningObject.Step) / _size_y) + 1] == _barrier)
                            return false;
                    }
                    break;
            }
            return true;
        }
        public Bitmap MoveObject(Direction direction)
        {
            if (CheckMove(direction))
            {
                _drawningObject.MoveObject(direction);
            }
            return DrawMapWithObject();
        }
        private bool SetObjectOnMap()
        {
            if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 10);
            int y = _random.Next(0, 10);
            _drawningObject.SetObject(x, y, _width, _height);
            for (int i = GetCoord().LeftCoord; i <= GetCoord().RightCoord; i++)
            {
                for (int j = GetCoord().TopCoord; j <= GetCoord().BottomCoord; j++)
                {
                    if (_map[i, j] == _barrier)
                    {
                        return false;
                    }
                }
            }
            return true;
        }
        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (_map[i, j] == _freeRoad)
                    {
                        DrawRoadPart(gr, i, j);
                    }
                    else if (_map[i, j] == _barrier)
                    {
                        DrawBarrierPart(gr, i, j);
                    }
                }
            }
            _drawningObject.DrawningObject(gr);
            return bmp;
        }

        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}